#include <iostream>
//#include <string>
#include <sstream>
#include <cmath>

using namespace std;


// Constants & variables
string verInfo = "4wins v0.2 by Lonard";
string verInfoBot = "Bot v0.04";

const int MAX_COLUMNS   = 7;
const int MAX_ROWS      = 6;

short field[MAX_COLUMNS][MAX_ROWS];
short fieldAfterMove[MAX_COLUMNS][MAX_ROWS];

short playerCPU     = -1;
short playerHuman1  = -1;
short playerHuman2  = -1;


// Function declarations
void drawField();
bool getField(string);
bool moveIsValid(short);
short getMove(short);
void setMove(short, short);
bool checkFor4(short);
bool checkForDraw();

// Game modes
void modePVP();
void modeStandAlone();
void modeBotOnly();

// Bot functions
int botNextMove();
bool checkFieldAfterMove(short, short);
string checkFor3(short, short = 0);
string checkFor2Plus1(short, short = 0);
string checkFor2(short);
string checkFor1Plus1(short);
string strategy(short);
string tryZwickmuehle(short);
int zwickmuehlePossibleAt(short, int, int, int);
int buildZwickmuehle(short, int, int, int, int);


// Functions
int main()
{

    // Get game mode
    string tmp;
    short gameMode;
    stringstream converter;

    while (true)
    {
        converter = stringstream();
        cout << endl << endl << "  " << verInfo << endl << endl <<
            "  [1] player vs. player" << endl <<
            "  [2] player vs. bot" << endl <<
            "  [3] bot only" << endl << endl <<
            "  Select game mode: ";

        getline(cin, tmp);
        converter << tmp;
        if ( (converter >> gameMode) && (gameMode >= 1) && (gameMode <= 3) )
        {
            break;
        }
        else
        {
            cout << "  Invalid input! Try again!" << endl;
        }
    }

    switch (gameMode)
    {
    case 1:
        modePVP();
        break;

    case 2:
        modeStandAlone();
        break;

    case 3:
        modeBotOnly();
        break;

    default:
        cout << endl << "  Invalid input!" << endl << endl;
        return 1;
        break;
    }

    return 0;

}

void modePVP()
{

    string tmp;

    playerHuman1 = 1;
    playerHuman2 = 2;

    drawField();

    // Game loop
    while (true)
    {
        setMove( playerHuman1, getMove(playerHuman1) );
        drawField();
        if ( checkFor4(playerHuman1) )
        {
            cout << "  Congratulations, player 1! You win! ";
            getline(cin, tmp);
            return;
        }
        if ( checkForDraw() )
        {
            cout << "  Draw! ";
            getline(cin, tmp);
            return;
        }

        setMove( playerHuman2, getMove(playerHuman2) );
        drawField();
        if ( checkFor4(playerHuman2) )
        {
            cout << "  Congratulations, player 2! You win! ";
            getline(cin, tmp);
            return;
        }
        if ( checkForDraw() )
        {
            cout << "  Draw! ";
            getline(cin, tmp);
            return;
        }
    }

}

void modeStandAlone()
{

    /*
     * TODO:
     * use botNextMove() for modeBotOnly()
     */

    // Get player number
    string tmp = "";
    stringstream converter;
    short nextMove;

    while (true)
    {
        converter = stringstream();
        cout << endl <<
            "  Will you be player [1] or [2]? ";
        getline(cin, tmp);
        converter << tmp;
        if ( (converter >> playerHuman1) && (playerHuman1 >= 1) && (playerHuman1 <= 2) )
        {
            break;
        }
        else
        {
            cout << "  Invalid input! Try again." << endl;
        }
    }

    playerCPU = (playerHuman1 == 1) ? 2 : 1;
    cout << endl << endl;

    // If human player is player 1, his will be the first turn
    if (playerHuman1 == 1)
    {
        drawField();
        // Get and set input from player
        setMove( playerHuman1, getMove(playerHuman1) );
    }

    // Game loop
    while (true)
    {

        drawField();

        // Did the human player win with his last move?
        if ( checkFor4(playerHuman1) )
        {
            cout << "  Congratulations! You win! ";
            getline(cin, tmp);
            return;
        }

        if ( checkForDraw() )
        {
            cout << "  Draw! ";
            getline(cin, tmp);
            return;
        }

        // What will my next move be?
        nextMove = botNextMove();
        if (nextMove != -1)
        {
            cout << "  My next move will be: " << to_string(nextMove) << " ";
            setMove(playerCPU, nextMove);
        }

        /* *** BOT: OLD VERSION ***
         *
        // Check if I can win anywhere with 3 similar stones in a row or column
        tmp = checkFor3(playerCPU);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: checkFor3(playerCPU)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Check if I can win anywhere with 2 similar stones in a row and a single stone, separated by one free field
        tmp = checkFor2Plus1(playerCPU);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: checkFor2Plus1(playerCPU)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Check if the human player can win if I don't intervene
        tmp = checkFor3(playerHuman1);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: checkFor3(playerHuman1)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Check if the human player can win if I don't intervene
        tmp = checkFor2Plus1(playerHuman1);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: checkFor2Plus1(playerHuman1)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Could I build up a row?
        tmp = checkFor1Plus1(playerCPU);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: checkFor1Plus1(playerCPU)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Check if the human player could build up a row
        tmp = checkFor1Plus1(playerHuman1);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: checkFor1Plus1(playerHuman1)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Could I build up a zwickmuehle?
        tmp = tryZwickmuehle(playerCPU);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: tryZwickmuehle(playerCPU)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Could I build up a row?
        tmp = checkFor2(playerCPU);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: checkFor2(playerCPU)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Check if the human player is building up a row
        tmp = checkFor2(playerHuman1);
        if ( (tmp[0] != 'f') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: checkFor2(playerHuman1)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }

        // Follow my 'special' strategy
        tmp = strategy(playerCPU);
        if ( (tmp[0] == 't') &! (cpuTurnFinished) )
        {
            cout << "  My next move will be: " << tmp.substr(1) << " ";
            converter << tmp.substr(1);
            converter >> nextMove;
            if (!moveIsValid(nextMove))
                cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                                "    >> function: strategy(playerCPU)" << endl;
            setMove(playerCPU, nextMove);
            cpuTurnFinished = true;
        }
        */

        // Pause and wait for user input
        getline(cin, tmp);
        drawField();

        if ( checkForDraw() )
        {
            cout << "  Draw! ";
            getline(cin, tmp);
            return;
        }

        // Did I win with my last move?
        if ( checkFor4(playerCPU) )
        {
            cout << "  I win :-) ";
            getline(cin, tmp);
            return;
        }

        // Get and set input from player
        setMove( playerHuman1, getMove(playerHuman1) );

    }

}

void modeBotOnly()
{

    // Get player number
    string tmp = "";
    stringstream converter;
    short nextMove;

    while (true)
    {
        cout << endl <<
            "  Am I player [1] or [2]? ";
        getline(cin, tmp);
        converter = stringstream();
        converter << tmp;
        if ( (converter >> playerCPU) && (playerCPU >= 1) && (playerCPU <= 2) )
        {
            break;
        }
        else
        {
            cout << "  Invalid input! Try again." << endl << "  ";
        }
    }

    playerHuman1 = (playerCPU == 1) ? 2 : 1;

    // Game loop
    while (true)
    {
        // Get input
        string input = "";
        cout << "  Enter field matrix [column1; column2; ...]: ";
        getline(cin, input);

        if (!getField(input))
        {
            cout << endl << "  Invalid input! ";
            return;
        }

        drawField();

        cout << endl << endl;

        // Did I win (or lose) already?
        if ( checkFor4(playerCPU) )
        {
            cout << "  I win :-)" << endl << endl;
            return;
        }
        if ( checkFor4(playerHuman1) )
        {
            cout << "  I lose :-<" << endl << endl;
            return;
        }

        cout << "  Scanning for possible moves..." << endl;

        nextMove = botNextMove();
        if (nextMove != -1)
        {
            cout << "  My next move will be: " << to_string(nextMove) << endl << endl;
        }
        else
        {
            cout << "  I can't think of anything to do. Did the game end in a draw?";
        }

        /* *** BOT: OLD VERSION ***
        // Check if I can win anywhere with 3 similar stones in a row or column
        tmp = checkFor3(playerCPU);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Check if I can win anywhere with 2 similar stones in a row and a single stone, separated by one free field
        tmp = checkFor2Plus1(playerCPU);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Check if the opponent can win if I don't intervene
        tmp = checkFor3(playerHuman1);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Check if the opponent can win if I don't intervene
        tmp = checkFor2Plus1(playerHuman1);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Could I build up a row?
        tmp = checkFor1Plus1(playerCPU);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Check if the opponent could build up a row
        tmp = checkFor1Plus1(playerHuman1);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Could I build up a zwickmuehle?
        tmp = tryZwickmuehle(playerCPU);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Could I build up a row?
        tmp = checkFor2(playerCPU);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Check if the opponent is building up a row
        tmp = checkFor2(playerHuman1);
        if (tmp[0] != 'f')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }

        // Follow my 'special' strategy
        tmp = strategy(playerCPU);
        if (tmp[0] == 't')
        {
            cout << "  My next move will be: " << tmp.substr(1) << endl << endl;
            continue;
        }
        else
        {
            cout << "  I can't think of anything to do. Did the game end in a draw?" << endl << endl;
            continue;
        }
        */
    }

}

int botNextMove()
{

    string tmp;
    stringstream converter;
    int nextMove;

    // Check if I can win anywhere with 3 similar stones in a row or column
    tmp = checkFor3(playerCPU);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: checkFor3(CPU)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: checkFor3(playerCPU)" << endl;
        return nextMove;
    }

    // Check if I can win anywhere with 2 similar stones in a row and a single stone, separated by one free field
    tmp = checkFor2Plus1(playerCPU);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: checkFor2Plus1(CPU)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: checkFor2Plus1(playerCPU)" << endl;
        return nextMove;
    }

    // Check if the human player can win if I don't intervene
    tmp = checkFor3(playerHuman1);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: checkFor3(Human)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: checkFor3(playerHuman1)" << endl;
        return nextMove;
    }

    // Check if the human player can win if I don't intervene
    tmp = checkFor2Plus1(playerHuman1);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: checkFor2Plus1(Human)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: checkFor2Plus1(playerHuman1)" << endl;
        return nextMove;
    }

    // Could I build up a row?
    tmp = checkFor1Plus1(playerCPU);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: checkFor1Plus1(CPU)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: checkFor1Plus1(playerCPU)" << endl;
        return nextMove;
    }

    // Check if the human player could build up a row
    tmp = checkFor1Plus1(playerHuman1);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: checkFor1Plus1(Humand)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: checkFor1Plus1(playerHuman1)" << endl;
        return nextMove;
    }

    // Could I build up a zwickmuehle?
    tmp = tryZwickmuehle(playerCPU);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: tryZwickmuehle(CPU)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: tryZwickmuehle(playerCPU)" << endl;
        return nextMove;
    }

    // Could I build up a row?
    tmp = checkFor2(playerCPU);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: checkFor2(CPU)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: checkFor2(playerCPU)" << endl;
        return nextMove;
    }

    // Check if the human player is building up a row
    tmp = checkFor2(playerHuman1);
    if (tmp[0] != 'f')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: checkFor1(CPU)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: checkFor2(playerHuman1)" << endl;
        return nextMove;
    }

    // Follow my 'special' strategy
    tmp = strategy(playerCPU);
    if (tmp[0] == 't')
    {
        converter << tmp.substr(1);
        converter >> nextMove;

        // +++++ DEBUG +++++
        cout << endl << "  >> function: strategy(CPU)" << endl;
        // +++++ DEBUG +++++

        if (!moveIsValid(nextMove))
            cout << endl << "    >> THIS MOVE IS INVALID! Please take a screenshot and report this bug to the developer. " << endl <<
                            "    >> function: strategy(playerCPU)" << endl;
        return nextMove;
    }

    // Can't think of anything to do. Did the game end in a draw?
    return -1;

}

string checkFor2(short playerNumber)
{

    // Check for 2 stones in a row that can be built up into a row of 3
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            if (field[x][y] == playerNumber)
            {
                if ( (x < MAX_COLUMNS - 1) && (field[x + 1][y] == playerNumber) )
                {
                    if ( (x < MAX_COLUMNS - 3) && (field[x + 2][y] == 0) && (field[x + 3][y] == 0) )
                    {
                        if ( (y == MAX_ROWS - 1) || (field[x + 2][y + 1] != 0) )
                        {
                            // IF there are 2 stones in a row AND the 2 fields to the right are free AND we're in the bottom row
                            // OR the field below the free field is not free
                            // return "t(rue)x", x being my next move
                            // But first: check if this would mean a chance to win for the opponent!
                            if ( checkFieldAfterMove(x + 2, playerNumber) )
                                return "t" + to_string(x + 2);
                        }
                    }
                    if ( (x > 1) && (field[x - 1][y] == 0) && (field[x - 2][y] == 0) )
                    {
                        if ( (y == MAX_ROWS - 1) || (field[x - 1][y + 1] != 0) )
                        {
                            // IF there are 2 stones in a row AND 2 fields to the left are free
                            // AND we're in the bottom row OR the field below the free field is not free
                            // return "t(rue)x", x being my next move
                            // But first: check if this would mean a chance to win for the opponent!
                            if ( checkFieldAfterMove(x - 1, playerNumber) )
                                return "t" + to_string(x - 1);
                        }
                    }
                    if ( (x > 0) && (field[x - 1][y] == 0) && (x < MAX_COLUMNS - 2) && (field[x + 2][y] == 0) )
                    {
                        if ( (y == MAX_ROWS - 1) || (field[x - 1][y + 1] != 0) )
                        {
                            // IF there are 2 stones in a row AND one field to the left + one field to the right are free
                            // AND we're in the bottom row OR the field below the free field is not free
                            // return "t(rue)x", x being my next move
                            // But first: check if this would mean a chance to win for the opponent!
                            if ( checkFieldAfterMove(x - 1, playerNumber) )
                                return "t" + to_string(x - 1);
                        }
                        if ( (y == MAX_ROWS - 1) || (field[x + 2][y + 1] != 0) )
                        {
                            // IF there are 2 stones in a row AND the one field to the right are free AND we're in the bottom row
                            // OR the field below the free field is not free
                            // return "t(rue)x", x being my next move
                            // But first: check if this would mean a chance to win for the opponent!
                            if ( checkFieldAfterMove(x + 2, playerNumber) )
                                return "t" + to_string(x + 2);
                        }
                    }
                }
            }
        }
    }

    // Return "f(alse)" for no chance to win found
    return "f";

}

string checkFor1Plus1(short playerNumber)
{

    // Check for 1 stone, 1 free field, 1 stone in a row that can be built up into a row of 3
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            if (field[x][y] == playerNumber)
            {
                if ( (x < MAX_COLUMNS - 1) && (field[x + 1][y] == 0) )
                {
                    if ( (x < MAX_COLUMNS - 2) && (field[x + 2][y] == playerNumber) )
                    {
                        if ( (x < MAX_COLUMNS - 3) && (field[x + 3][y] == 0) )
                        {
                            if ( (y == MAX_ROWS - 1) || (field[x + 1][y + 1] != 0) )
                            {
                                // IF we have 1 stone, 1 field free, 1 stone in a row AND the next field is free AND we're in the bottom row
                                // OR the field below the free field is not free
                                // return "t(rue)x", x being my next move
                                // But first: check if this would mean a chance to win for the opponent!
                                if ( checkFieldAfterMove(x + 1, playerNumber) )
                                    return "t" + to_string(x + 1);
                            }
                            if ( (y == MAX_ROWS - 1) || (field[x + 3][y + 1] != 0) )
                            {
                                // IF we have 1 stone, 1 field free, 1 stone in a row AND the next field is free AND we're in the bottom row
                                // OR the field below the free field is not free
                                // return "t(rue)x", x being my next move
                                // But first: check if this would mean a chance to win for the opponent!
                                if ( checkFieldAfterMove(x + 3, playerNumber) )
                                    return "t" + to_string(x + 3);
                            }
                        }
                        if ( (x > 0) && (field[x - 1][y] == 0) )
                        {
                            if ( (y == MAX_ROWS - 1) || (field[x + 1][y + 1] != 0) )
                            {
                                // IF we have 1 stone, 1 field free, 1 stone in a row AND the next field is free AND we're in the bottom row
                                // OR the field below the free field is not free
                                // return "t(rue)x", x being my next move
                                // But first: check if this would mean a chance to win for the opponent!
                                if ( checkFieldAfterMove(x + 1, playerNumber) )
                                    return "t" + to_string(x + 1);
                            }
                            if ( (y == MAX_ROWS - 1) || (field[x - 1][y + 1] != 0) )
                            {
                                // IF we have 1 stone, 1 field free, 1 stone in a row AND one field to the left is free AND we're in the bottom row
                                // OR the field below the free field is not free
                                // return "t(rue)x", x being my next move
                                // But first: check if this would mean a chance to win for the opponent!
                                if ( checkFieldAfterMove(x - 1, playerNumber) )
                                    return "t" + to_string(x - 1);
                            }
                        }
                    }
                }
            }
        }
    }

    // Return "f(alse)" for no chance to win found
    return "f";

}

string checkFor2Plus1(short playerNumber, short checkFieldAfterNextMove)
{

    // Are we checking for what happens after my next move?
    short fieldToCheck[MAX_COLUMNS][MAX_ROWS];

    if ( checkFieldAfterNextMove != 0)
    {
        for (int x = 0; x < MAX_COLUMNS; x++)
        {
            for (int y = 0; y < MAX_ROWS; y++)
            {
                fieldToCheck[x][y] = fieldAfterMove[x][y];
            }
        }
    }
    else
    {
        for (int x = 0; x < MAX_COLUMNS; x++)
        {
            for (int y = 0; y < MAX_ROWS; y++)
            {
                fieldToCheck[x][y] = field[x][y];
            }
        }
    }

    // Check for 2 stones in a row and a single one with one free field in between
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            if (fieldToCheck[x][y] == playerNumber)
            {
                if ( (x < MAX_COLUMNS - 1) && (fieldToCheck[x + 1][y] == playerNumber) )
                {
                    if ( (x < MAX_COLUMNS - 2) && (fieldToCheck[x + 2][y] == 0) )
                    {
                        if ( (x < MAX_COLUMNS -3) && (fieldToCheck[x + 3][y] == playerNumber) )
                        {
                            if ( (y == MAX_ROWS - 1) || (fieldToCheck[x + 2][y + 1] != 0) )
                            {
                                // IF there are 2 stones in a row AND next field free AND next field containing
                                // one of my stones AND we're in the bottom row OR the field below the free field is not free
                                // return "t(rue)x", x being my next move
                                return "t" + to_string(x + 2);
                            }
                        }
                    }
                    if ( (x > 0) && (fieldToCheck[x - 1][y] == 0) )
                    {
                        if ( (x > 1) && (fieldToCheck[x - 2][y] == playerNumber) )
                        {
                            if ( (y == MAX_ROWS - 1) || (fieldToCheck[x - 1][y + 1] != 0) )
                            {
                                // IF there are 2 stones in a row AND one field to the left is free AND one further to the left containing
                                // one of my stones AND we're in the bottom row OR the field below the free field is not free
                                // return "t(rue)x", x being my next move
                                return "t" + to_string(x - 1);
                            }
                        }
                    }
                }
            }
        }
    }

    // Return "f(alse)" for no chance to win found
    return "f";

}

string checkFor3(short playerNumber, short checkFieldAfterNextMove)
{

    // Are we checking for what happens after my next move?
    short fieldToCheck[MAX_COLUMNS][MAX_ROWS];

    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            if (checkFieldAfterNextMove != 0)
            {
                fieldToCheck[x][y] = fieldAfterMove[x][y];
            }
            else
            {
                fieldToCheck[x][y] = field[x][y];
            }
        }
    }

    // Check for 3 stones in a row
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            if (fieldToCheck[x][y] == playerNumber)
            {
                if ( (x < MAX_COLUMNS - 1) && (fieldToCheck[x + 1][y] == playerNumber) )
                {
                    if ( (x < MAX_COLUMNS - 2) && (fieldToCheck[x + 2][y] == playerNumber) )
                    {
                        if ( (x < MAX_COLUMNS - 3) && (fieldToCheck[x + 3][y] == 0) )
                        {
                            if ( (y == MAX_ROWS - 1) || (fieldToCheck[x + 3][y + 1] != 0) )
                            {
                                // IF 3 stones in a row PLUS next field is still within the field PLUS is free PLUS we're in the
                                // bottom-most row OR one field lower is not free
                                // return "t(rue)x", x being my next move
                                return "t" + to_string(x + 3);
                            }
                        }
                        if ( (x > 0) && (fieldToCheck[x - 1][y] == 0) )
                        {
                            if ( (y == MAX_ROWS - 1) || (fieldToCheck[x - 1][y + 1] != 0) )
                            {
                                // IF 3 stones in a row PLUS one field to the left is still within the field PLUS is free PLUS we're in the
                                // bottom-most row OR one field lower is not free
                                // return "t(rue)x", x being my next move
                                return "t" + to_string(x - 1);
                            }
                        }
                    }
                }
            }
        }
    }

    // Check for 3 stones in a column
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            if (fieldToCheck[x][y] == playerNumber)
            {
                if ( (y < MAX_ROWS - 1) && (fieldToCheck[x][y + 1] == playerNumber) )
                {
                    if ( (y < MAX_ROWS - 2) && (fieldToCheck[x][y + 2] == playerNumber) )
                    {
                        if ( (y > 0) && (fieldToCheck[x][y - 1] == 0) )
                        {
                            // IF 3 stones in a column PLUS field above is still within field PLUS is free
                            // return "t(rue)x", x being my next move
                            return "t" + to_string(x);
                        }
                    }
                }
            }
        }
    }

    // Return "f(alse)" for no chance to win found
    return "f";

}

bool checkFor4(short playerNumber)
{

    // Check for 4 similar stones in a row
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            if (field[x][y] == playerNumber)
            {
                if ( (x < MAX_COLUMNS -1) && (field[x + 1][y] == playerNumber) )
                {
                    if ( (x < MAX_COLUMNS - 2) && (field[x + 2][y] == playerNumber) )
                    {
                        if ( (x < MAX_COLUMNS - 3) && (field[x + 3][y] == playerNumber) )
                        {
                            return true;
                        }
                    }
                }
            }
        }
    }

    // Check for 4 similar stones in a column
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            if (field[x][y] == playerNumber)
            {
                if ( (y < MAX_ROWS -1) && (field[x][y + 1] == playerNumber) )
                {
                    if ( (y < MAX_ROWS - 2) && (field[x][y + 2] == playerNumber) )
                    {
                        if ( (y < MAX_ROWS - 3) && (field[x][y + 3] == playerNumber) )
                        {
                            return true;
                        }
                    }
                }
            }
        }
    }

    return false;

}

bool checkForDraw()
{

    // Are there any free fields left or did the game end in a draw?
    bool gameEndedDraw = true;
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        if (field[x][0] == 0)
            gameEndedDraw = false;
    }

    return gameEndedDraw;

}

bool checkFieldAfterMove(short nextMove, short playerNumber)
{

    // Check if my next move would mean a chance to win for the opponent
    for (int x = 0; x < MAX_COLUMNS; x++)
    {
        for (int y = 0; y < MAX_ROWS; y++)
        {
            fieldAfterMove[x][y] = field[x][y];
        }
    }

    for (int y = MAX_ROWS - 1; y >= 0; y--)
    {
        if (fieldAfterMove[nextMove][y] == 0)
        {
            fieldAfterMove[nextMove][y] = playerNumber;
            break;
        }
    }

    short playerToCheck = (playerNumber == 1) ? 2 : 1;

    if ( checkFor2Plus1(playerToCheck, 1)[0] != 'f' )
        return false;
    if ( checkFor3(playerToCheck, 1)[0] != 'f' )
        return false;

    return true;

}

string tryZwickmuehle(short playerNumber)
{

    // Is the field large enough for this kind of zwickmuehle?
    if (MAX_COLUMNS < 7)
        return "f";

    int middleColumn    = round(MAX_COLUMNS / 2.0) - 1;
    int possibleAtY     = -1;
    int tmp;

    // Check Zwickmühle " x - x x - x - "
    //                   (0 1 2 3 4 5 6)
    possibleAtY = zwickmuehlePossibleAt(playerNumber, middleColumn, middleColumn - 3, middleColumn + 2);
    if (possibleAtY != -1)
    {
        tmp = buildZwickmuehle(playerNumber, possibleAtY, middleColumn - 3, middleColumn + 2, middleColumn - 1);
        if (tmp != -1)
            return "t" + to_string(tmp);
    }

    // Check Zwickmühle " - x - x x - x "
    //                   (0 1 2 3 4 5 6)
    possibleAtY = zwickmuehlePossibleAt(playerNumber, middleColumn, middleColumn - 2, middleColumn + 3);
    if (possibleAtY != -1)
    {
        tmp = buildZwickmuehle(playerNumber, possibleAtY, middleColumn + 3, middleColumn - 2, middleColumn + 1);
        if (tmp != -1)
            return "t" + to_string(tmp);
    }

    return "f";

}

int zwickmuehlePossibleAt(short playerNumber, int middleColumn, int leftBoundary, int rightBoundary)
{

    bool zwickmuehlePossible;

    for (int y = MAX_ROWS - 1; y >= 0; y--)
    {
        zwickmuehlePossible = true;
        // Do I have a stone in the middle column to work with?
        if (field[middleColumn][y] == playerNumber)
        {
            // Is there enough space left to the left to build up my zwickmuehle?
            for (int x = middleColumn; x >= leftBoundary; x--)
            {
                if ( (field[x][y] != 0) && (field[x][y] != playerNumber) )
                {
                    zwickmuehlePossible = false;
                    break;
                }
            }
            if (zwickmuehlePossible == true)
            {
                // Is there enough space left to the right to build up my zwickmuehle?
                for (int x = middleColumn; x <= rightBoundary; x++)
                {
                    if ( (field[x][y] != 0) && (field[x][y] != playerNumber) )
                    {
                        zwickmuehlePossible = false;
                        break;
                    }
                }
                if (zwickmuehlePossible == true)
                {
                    // It's possible to build up a zwickmuehle in this row.
                    return y;
                    break;
                }
            }
        }
    }

    return -1;

}

int buildZwickmuehle(short playerNumber, int y, int move1, int move2, int move3)
{

    // Check if the needed fields are free. If so, put a stone there.
    short nextMove[3] = {static_cast<short>(move1), static_cast<short>(move2), static_cast<short>(move3)};
    for (unsigned int i = 0; i < sizeof(nextMove) / sizeof(short); i++)
    {
        if (field[nextMove[i]][y] == 0)
        {
            if ( (y == MAX_ROWS - 1) || (field[nextMove[i]][y + 1] != 0) )
            {
                // IF this wouldn't mean a winning option for the opponent, build up the zwickmuehle.
                if ( checkFieldAfterMove(nextMove[i], playerNumber) )
                    return nextMove[i];
            }
        }
    }

    return -1;

}


string strategy(short playerNumber)
{

    // Fill the field up from the middle column as this creates the most potential chances for winning

    short middleColumn = round(MAX_COLUMNS / 2.0) - 1;
    short nextMove;

    // Get some stones into the middle column first
    nextMove = middleColumn;
    if ( (field[nextMove][3] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
    {
        // IF (x columns right of) the middle column has more than 3 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
        // return "t(rue)x", x being my next move
        return "t" + to_string(nextMove);
    }

    // Check every other column right and left of the middle column and fill them up
    for (int x = 0; x < MAX_COLUMNS - middleColumn; x += 2)
    {
        nextMove = middleColumn + x;
        if ( (field[nextMove][3] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns right of) the middle column has more than 3 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
        nextMove = middleColumn - x;
        if ( (field[nextMove][3] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns left of) the middle column has more than 3 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
    }

    // Check every single column and fill them up
    for (int x = 0; x < MAX_COLUMNS - middleColumn; x++)
    {
        nextMove = middleColumn + x;
        if ( (field[nextMove][3] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns right of) the middle column has more than 3 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
        nextMove = middleColumn - x;
        if ( (field[nextMove][3] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns left of) the middle column has more than 3 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
    }

    // Fill up every other column further
    for (int x = 0; x < MAX_COLUMNS - middleColumn; x += 2)
    {
        nextMove = middleColumn + x;
        if ( (field[nextMove][2] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns right of) the middle column has more than 2 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
        nextMove = middleColumn - x;
        if ( (field[nextMove][2] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns left of) the middle column has more than 2 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
    }

    // Fill up every single column further
    for (int x = 0; x < MAX_COLUMNS - middleColumn; x++)
    {
        nextMove = middleColumn + x;
        if ( (field[nextMove][2] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns right of) the middle column has more than 2 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
        nextMove = middleColumn - x;
        if ( (field[nextMove][2] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns left of) the middle column has more than 2 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
    }

    // Fill up every other column further
    for (int x = 0; x < MAX_COLUMNS - middleColumn; x += 2)
    {
        nextMove = middleColumn + x;
        if ( (field[nextMove][1] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns right of) the middle column has more than 1 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
        nextMove = middleColumn - x;
        if ( (field[nextMove][1] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns left of) the middle column has more than 1 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
    }

    // Fill up every single column further
    for (int x = 0; x < MAX_COLUMNS - middleColumn; x++)
    {
        nextMove = middleColumn + x;
        if ( (field[nextMove][1] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns right of) the middle column has more than 1 free field left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
        nextMove = middleColumn - x;
        if ( (field[nextMove][1] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns left of) the middle column has more than 1 free field left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
    }

    // Fill up every other column to the top
    for (int x = 0; x < MAX_COLUMNS - middleColumn; x += 2)
    {
        nextMove = middleColumn + x;
        if ( (field[nextMove][0] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns right of) the middle column has more than 0 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
        nextMove = middleColumn - x;
        if ( (field[nextMove][0] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns left of) the middle column has more than 0 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
    }

    // Fill up every single column to the top
    for (int x = 0; x < MAX_COLUMNS - middleColumn; x++)
    {
        nextMove = middleColumn + x;
        if ( (field[nextMove][0] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns right of) the middle column has more than 0 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
        nextMove = middleColumn - x;
        if ( (field[nextMove][0] == 0) && (checkFieldAfterMove(nextMove, playerNumber)) )
        {
            // IF (x columns left of) the middle column has more than 0 free fields left AND putting a stone there wouldn't mean a win situation for the opponent
            // return "t(rue)x", x being my next move
            return "t" + to_string(nextMove);
        }
    }

    for (int x = 0; x < MAX_COLUMNS - middleColumn; x++)
    {
        nextMove = middleColumn + x;
        if (field[nextMove][0] == 0)
            // If nothing else works, just put the goddamn stone anywhere free
            return "t" + to_string(nextMove);

        nextMove = middleColumn - x;
        if (field[nextMove][0] == 0)
            // If nothing else works, just put the goddamn stone anywhere free
            return "t" + to_string(nextMove);
    }

    return "f";

}

bool moveIsValid(short nextMove)
{

    if ( (nextMove >= 0) && (nextMove <= MAX_COLUMNS - 1) && (field[nextMove][0] == 0) )
    {
        return true;
    }

    return false;

}

short getMove(short playerNumber)
{

    stringstream converter;
    string tmp;
    short nextMove;

    while (true)
    {
        converter = stringstream();
        cout << "  Enter next move, player " << to_string(playerNumber) << ": ";
        getline(cin, tmp);
        converter << tmp;
        if ( (converter >> nextMove) && (moveIsValid(nextMove)) )
        {
            break;
        }
        else
        {
            cout << "  Invalid input! Try again." << endl;
        }
    }

    return nextMove;

}

void setMove(short playerNumber, short nextMove)
{

    for (int y = MAX_ROWS - 1; y >= 0; y--)
    {
        if (field[nextMove][y] == 0)
        {
            field[nextMove][y] = playerNumber;
            break;
        }
    }

}

void drawField()
{

    cout << endl << endl  << endl << endl << endl << endl << endl <<
            "  [" << verInfo << " - " << verInfoBot << "]" << endl << endl;

    for (int y = 0; y <= MAX_ROWS; y++)
    {
        cout << "   ";
        for (int x = 0; x < MAX_COLUMNS; x++)
        {
            if (y == MAX_ROWS)
            {
                cout << to_string(x);
            }
            else
            {
                switch (field[x][y])
                {
                case 0:
                    cout << "x";
                    break;

                default:
                    cout << to_string(field[x][y]);
                    break;
                }
            }
            cout << "      ";
        }
        cout << endl << endl << endl;
    }

}

bool getField(string input)
{

    // Parse input (array-string) to field[][]
    int x  = 0;
    int y  = 0;

    stringstream converter;

    for (int i = 0; i < static_cast<int>(input.length()); i++)
    {

        switch (input[i])
        {
            case '[':
            case ']':
            case ' ':
                break;

            case ',':
                y += 1;
                break;

            case ';':
                y  = 0;
                x  += 1;
                break;

            case '0':
            case '1':
            case '2':
                converter << input[i];
                converter >> field[x][y];
                converter.clear();
                break;

            default:
                return false;
                break;
        }

    }

    return true;

}
